@@ -1,4 +1,3 @@
-
 /*
  * $Id$
  *
@@ -68,6 +67,14 @@ public:
     /// [re]sets Content-Length header and cached value
     void setContentLength(int64_t clen);
 
+    /**
+     * \retval true  the message sender asks to keep the connection open.
+     * \retval false the message sender will close the connection.
+     *
+     * Factors other than the headers may result in connection closure.
+     */
+    bool persistent() const;
+
 public:
     HttpVersion http_ver;
 
@@ -82,7 +89,7 @@ public:
 
     int64_t content_length;
 
-    protocol_t protocol;
+    AnyP::ProtocolType protocol;
 
     HttpMsgParseState pstate;   /* the current parsing state */
 
@@ -129,6 +136,13 @@ protected:
 class HttpParser
 {
 public:
+    HttpParser() { clear(); }
+    HttpParser(const char *buf, int len);
+
+    /// Set this parser back to a default state.
+    /// Will DROP any reference to a buffer (does not free).
+    void clear();
+
     /**
      * Attempt to parse the first line of a new request message.
      *
@@ -156,6 +170,11 @@ public:
     int u_start, u_end;
     int v_start, v_end;
     int v_maj, v_min;
+
+    /** HTTP status code to be used on the invalid-request error page
+     * HTTP_STATUS_NONE indicates incomplete parse, HTTP_OK indicates no error.
+     */
+    http_status request_parse_status;
 };
 
 extern void HttpParserInit(HttpParser *, const char *buf, int len);
@@ -208,8 +227,7 @@ public:
     const Msg &operator *() const { return *msg; }
     Msg *operator ->() { return msg; }
     const Msg *operator ->() const { return msg; }
-    operator Msg *() { return msg; }
-    operator const Msg *() const { return msg; }
+    operator Msg *() const { return msg; }
     // add more as needed
 
     /// public access for HttpMsgPointerT copying and assignment; avoid
