@@ -104,8 +104,18 @@ ACLRegexData::dump()
 {
     wordlist *W = NULL;
     relist *temp = data;
+    int flags = REG_EXTENDED | REG_NOSUB;
 
     while (temp != NULL) {
+        if (temp->flags != flags) {
+            if ((temp->flags&REG_ICASE) != 0) {
+                wordlistAdd(&W, "-i");
+            } else {
+                wordlistAdd(&W, "+i");
+            }
+            flags = temp->flags;
+        }
+
         wordlistAdd(&W, temp->pattern);
         temp = temp->next;
     }
@@ -145,6 +155,7 @@ aclParseRegexList(relist **curlist)
         }
 
         q = (relist *)memAllocate(MEM_RELIST);
+        q->flags = flags;
         q->pattern = xstrdup(t);
         q->regex = comp;
         *(Tail) = q;
