@@ -166,33 +166,32 @@ AuthDigestUserRequest::authenticate(HttpRequest * request, ConnStateData * conn,
     return;
 }
 
-int
+Auth::Direction
 AuthDigestUserRequest::module_direction()
 {
     if (user()->auth_type != Auth::AUTH_DIGEST)
-        return -2;
+        return Auth::CRED_ERROR;
 
     switch (user()->credentials()) {
 
     case Auth::Ok:
-        return 0;
+        return Auth::CRED_VALID;
 
     case Auth::Failed:
         /* send new challenge */
-        return 1;
+        return Auth::CRED_CHALLENGE;
 
     case Auth::Unchecked:
     case Auth::Pending:
-        return -1;
+        return Auth::CRED_LOOKUP;
 
     default:
-        return -2;
+        return Auth::CRED_ERROR;
     }
 }
 
-/* add the [proxy]authorisation header */
 void
-AuthDigestUserRequest::addHeader(HttpReply * rep, int accel)
+AuthDigestUserRequest::addAuthenticationInfoHeader(HttpReply * rep, int accel)
 {
     http_hdr_type type;
 
@@ -218,9 +217,8 @@ AuthDigestUserRequest::addHeader(HttpReply * rep, int accel)
 }
 
 #if WAITING_FOR_TE
-/** add the [proxy]authorisation header */
 void
-AuthDigestUserRequest::addTrailer(HttpReply * rep, int accel)
+AuthDigestUserRequest::addAuthenticationInfoTrailer(HttpReply * rep, int accel)
 {
     int type;
 
