@@ -34,6 +34,7 @@
  */
 
 #include "squid.h"
+#include "comm/Write.h"
 #include "errorpage.h"
 #include "Store.h"
 #include "HttpReply.h"
@@ -101,7 +102,10 @@ whoisStart(FwdState * fwd)
     String str_print=p->request->urlpath.substr(1,p->request->urlpath.size());
     snprintf(buf, l, SQUIDSTRINGPH"\r\n", SQUIDSTRINGPRINT(str_print));
 
-    comm_write(fd, buf, strlen(buf), whoisWriteComplete, p, NULL);
+    AsyncCall::Pointer call = commCbCall(5,5, "whoisWriteComplete",
+                                         CommIoCbPtrFun(whoisWriteComplete, p));
+
+    Comm::Write(fd, buf, strlen(buf), call, NULL);
     comm_read(fd, p->buf, BUFSIZ, whoisReadReply, p);
     commSetTimeout(fd, Config.Timeout.read, whoisTimeout, p);
 }
@@ -128,73 +132,62 @@ WhoisState::setReplyToOK(StoreEntry *sentry)
 {
     HttpReply *reply = new HttpReply;
     sentry->buffer();
-    HttpVersion version(1, 0);
-    reply->setHeaders(version, HTTP_OK, "Gatewaying", "text/plain", -1, -1, -2);
+    reply->setHeaders(HTTP_OK, "Gatewaying", "text/plain", -1, -1, -2);
     sentry->replaceHttpReply(reply);
 }
 
 void
 WhoisState::readReply (int fd, char *aBuffer, size_t aBufferLength, comm_err_t flag, int xerrno)
 {
-    int do_next_read = 0;
-
     /* Bail out early on COMM_ERR_CLOSING - close handlers will tidy up for us */
-
-    if (flag == COMM_ERR_CLOSING) {
+    if (flag == COMM_ERR_CLOSING)
         return;
-    }
 
     aBuffer[aBufferLength] = '\0';
     debugs(75, 3, "whoisReadReply: FD " << fd << " read " << aBufferLength << " bytes");
     debugs(75, 5, "{" << aBuffer << "}");
 
-    if (flag == COMM_OK && aBufferLength > 0) {
-        if (!dataWritten)
-            setReplyToOK(entry);
-
-        kb_incr(&statCounter.server.all.kbytes_in, aBufferLength);
-
-        kb_incr(&statCounter.server.http.kbytes_in, aBufferLength);
-
-        /* No range support, we always grab it all */
-        dataWritten = true;
-
-        entry->append(aBuffer, aBufferLength);
-
-        entry->flush();
-
-        do_next_read = 1;
-    } else if (flag != COMM_OK || aBufferLength < 0) {
+    if (flag != COMM_OK) {
         debugs(50, 2, "whoisReadReply: FD " << fd << ": read failure: " << xstrerror() << ".");
 
         if (ignoreErrno(errno)) {
-            do_next_read = 1;
+            comm_read(fd, aBuffer, BUFSIZ, whoisReadReply, this);
         } else {
             ErrorState *err;
             err = errorCon(ERR_READ_ERROR, HTTP_INTERNAL_SERVER_ERROR, fwd->request);
             err->xerrno = errno;
             fwd->fail(err);
             comm_close(fd);
-            do_next_read = 0;
         }
-    } else {
-        entry->timestampsSet();
-        entry->flush();
-
-        if (!EBIT_TEST(entry->flags, RELEASE_REQUEST))
-            entry->setPublicKey();
-
-        fwd->complete();
-
-        debugs(75, 3, "whoisReadReply: Done: " << entry->url()  );
-
-        comm_close(fd);
-
-        do_next_read = 0;
+        return;
     }
 
-    if (do_next_read)
+    if (aBufferLength > 0) {
+        if (!dataWritten)
+            setReplyToOK(entry);
+
+        kb_incr(&statCounter.server.all.kbytes_in, aBufferLength);
+        kb_incr(&statCounter.server.http.kbytes_in, aBufferLength);
+
+        /* No range support, we always grab it all */
+        dataWritten = true;
+        entry->append(aBuffer, aBufferLength);
+        entry->flush();
+
         comm_read(fd, aBuffer, BUFSIZ, whoisReadReply, this);
+        return;
+    }
+
+    /* no bytes read. stop reading */
+    entry->timestampsSet();
+    entry->flush();
+
+    if (!EBIT_TEST(entry->flags, RELEASE_REQUEST))
+        entry->setPublicKey();
+
+    fwd->complete();
+    debugs(75, 3, "whoisReadReply: Done: " << entry->url());
+    comm_close(fd);
 }
 
 static void
